import unittest

from almapiwrapper.users import User, NewUser, Fee
from almapiwrapper.record import JsonData
from almapiwrapper import config_log
import os

config_log("test.log")

if os.getcwd().endswith('test'):
    os.chdir('..')

class TestCreateUser(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        u = User('TestFeeUser1', 'UBS', 'S')

        if u.data is None:
            # Create new user
            data = JsonData(filepath='test/data/user_testFeeUser1.json')
            _ = NewUser('UBS', 'S', data).create()

    def test_create_fee(self):

        u = User('TestFeeUser1', 'UBS', 'S')

        # Create new fee
        data = JsonData(filepath='test/data/fee_test1.json')
        f1 = Fee(user=u, data=data, create_fee=True)

        self.assertFalse(f1.error, 'Error during fee creation')
        self.assertNotEqual(len(u.fees), 0, 'No fee created')

        # Check amount of the fee
        f2 = Fee(user=u, fee_id=f1.fee_id)
        self.assertEqual(f2.data['balance'], 20, 'Amount should be "20.0"')

        # pay the fee
        f2.operate('pay')
        self.assertEqual(f2.data['status']['value'], 'CLOSED', 'Unable to pay the fine')

    @classmethod
    def tearDownClass(cls):
        u = User('TestFeeUser1', 'UBS', 'S')
        if u.data is not None:
            for fee in u.fees:
                fee.operate('waive')
            u.delete()


if __name__ == '__main__':
    unittest.main()



import unittest
import sys
import os
import time
import pandas as pd

from almapiwrapper.config import RecSet, NewLogicalSet, NewItemizedSet, Job, Reminder, fetch_reminders
from almapiwrapper.record import JsonData, XmlData
from almapiwrapper import config_log

config_log("test.log")
if os.getcwd().endswith('test'):
    os.chdir('..')

class TestNewLogicalSet(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        RecSet(name='TEST_RRE_123', zone='NZ', env='S').delete()
        RecSet(name='TEST_RRE_123b', zone='NZ', env='S').delete()

    def test_create(self):
        s1 = NewLogicalSet('NZ',
                       'S',
                       'TEST_RRE_123',
                       'TEST_RRE_123',
                       'BIB_MMS where BIB_MMS ((all CONTAIN "991082448539705501"))',
                       'raphael.rey@slsp.ch',
                       True)
        s1 = s1.create()
        self.assertEqual(type(s1).__name__, 'LogicalSet')
        members = s1.get_members()

        self.assertEqual(len(members), 1, f'It should one member, {len(members)} available')

        self.assertEqual(s1.get_content_type(),
                         'BIB_MMS',
                         f'Content type should be "BIB_MMS", it is {s1.get_content_type()}.')

        s2 = RecSet(name='TEST_RRE_123', zone='NZ', env='S')

        members = s2.get_members()

        self.assertEqual(len(members), 1, f'It should one member, {len(members)} available')

        s2.delete()

    def test_create_2(self):
        data = XmlData(filepath='test/data/set_NZ_TEST_RRE_123b1.xml')
        s1 = NewLogicalSet('NZ',
                           'S',
                           data=data)
        s1 = s1.create()
        self.assertEqual(type(s1).__name__, 'LogicalSet')

        members = s1.get_members()

        self.assertEqual(len(members), 1, f'It should one member, {len(members)} available')


class TestNewItemizedSet(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        s1 = RecSet(zone='NZ',
                    env='S',
                    name='TEST_RRE_1234')
        s1.delete()
        s2 = RecSet(zone='NZ',
                    env='S',
                    name='TEST_RRE_1234_itemized')
        s2.delete()
        s3 = RecSet(zone='NZ',
                    env='S',
                    name='TEST_RRE_123456_itemized')
        s3.delete()


    def test_create(self):

        # Create a logical set
        s1 = NewLogicalSet('NZ',
                           'S',
                           'TEST_RRE_1234',
                           'TEST_RRE_123',
                           'BIB_MMS where BIB_MMS ((all CONTAIN "991082448539705501"))',
                           'raphael.rey@slsp.ch',
                           True)
        s1 = s1.create()
        self.assertEqual(type(s1).__name__, 'LogicalSet')
        members = s1.get_members()

        self.assertEqual(len(members), 1, f'It should one member, {len(members)} available')

        self.assertEqual(s1.get_content_type(),
                         'BIB_MMS',
                         f'Content type should be "BIB_MMS", it is {s1.get_content_type()}.')

        # Create an itemized set from a logical set
        s2 = NewItemizedSet(zone='NZ', env='S', from_logical_set=s1)
        s3 = s2.create()
        time.sleep(5)
        self.assertEqual(type(s3).__name__, 'ItemizedSet')

        members_2 = s3.get_members()

        self.assertEqual(len(members_2), 1, f'It should one member, {len(members_2)} available')

        self.assertEqual(s3.get_content_type(),
                         'BIB_MMS',
                         f'Content type should be "BIB_MMS", it is {s3.get_content_type()}.')

        s3.add_members(['991125596919705501'])
        members_3 = s3.get_members()

        self.assertEqual(len(members_3), 2, f'It should one member, {len(members_3)} available')

        s1.delete()
        s3.delete()

    def test_create_empty(self):
        # Create an empty itemized set
        s1 = NewItemizedSet(zone='NZ', env='S', content='BIB_MMS', name='TEST_RRE_123456_itemized')
        s1 = s1.create()

        mms_ids = ['991090586569705501', '991125596919705501']
        # mms_ids = pd.read_csv('test/data/recset_test_mms_ids.csv', dtype=str)['MMS ID'].tolist()
        s1.add_members(mms_ids, fail_on_invalid_id=False)

        s2 = RecSet(zone='NZ',
                    env='S',
                    name='TEST_RRE_123456_itemized')
        members = s2.get_members()
        self.assertEqual(len(members), len(mms_ids), f'It should {len(mms_ids)} members, {len(members)} available')

        s1.delete()


if __name__ == '__main__':
    unittest.main()



import unittest
import sys
import os
import time
import pandas as pd

from almapiwrapper.inventory import Item
from almapiwrapper.acquisitions import POLine, Vendor, Invoice, fetch_invoices
from almapiwrapper.record import JsonData, XmlData
from almapiwrapper import config_log

config_log("test.log")
if os.getcwd().endswith('test'):
    os.chdir('..')


class TestPOLine(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        item = Item(barcode='A1002624931', zone='UBS', env='S')
        if item.data.find('.//po_line') is not None:
            pol_number = item.data.find('.//po_line').text
            pol = POLine(pol_number, zone='UBS', env='S')
            _ = pol.data
        else:
            pol = None

        if pol is None or pol.error is True:
            data = JsonData(filepath='test/data/pol_test1.json')
            pol = POLine(data=data, zone='UBS', env='S').create()
            item = Item(barcode='A1002624931', zone='UBS', env='S')
            item.data.find('.//po_line').text = pol.pol_number
            item.update()

    def test_update(self):
        item = Item(barcode='A1002624931', zone='UBS', env='S')
        pol_number = item.data.find('.//po_line').text
        pol = POLine(pol_number, zone='UBS', env='S')
        reclaim_interval = pol.data['reclaim_interval']
        if reclaim_interval == '60':
            pol.data['reclaim_interval'] = '30'
        else:
            pol.data['reclaim_interval'] = '60'
        reclaim_interval = pol.data['reclaim_interval']
        pol.update()

        self.assertEqual(pol.data['reclaim_interval'],
                         reclaim_interval,
                         f'POL reclaim interval should be {reclaim_interval}')

    def test_get_items(self):
        item = Item(barcode='A1002624931', zone='UBS', env='S')
        pol_number = item.data.find('.//po_line').text
        pol = POLine(pol_number, zone='UBS', env='S')
        items = pol.get_items()
        self.assertNotEqual(len(items), 0, 'No item found')

        self.assertEqual(items[0].barcode, 'A1002624931', 'Item barcode should be A1111000')

    def test_get_vendor(self):
        item = Item(barcode='A1002624931', zone='UBS', env='S')
        pol_number = item.data.find('.//po_line').text
        pol = POLine(pol_number, zone='UBS', env='S')
        vendor = pol.get_vendor()
        self.assertIsInstance(vendor, Vendor, 'Vendor should be an instance of Vendor')
        self.assertEqual(vendor.vendor_code, 'A100-1043', 'Vendor code should be "A100-1043"')


if __name__ == '__main__':
    unittest.main()



import unittest

from almapiwrapper.analytics import AnalyticsReport
from almapiwrapper.record import JsonData
from almapiwrapper import config_log

config_log("test.log")


class TestgetAnalyticsReport(unittest.TestCase):

    def test_get_analytics_report(self):

        report = AnalyticsReport('/shared/SLSP Network Zone 41SLSP_NETWORK/Reports/RRE/Almapiwrapper_test', 'NZ', 'P')


        # Check path
        self.assertEqual(report.path,
                         '/shared/SLSP Network Zone 41SLSP_NETWORK/Reports/RRE/Almapiwrapper_test',
                         'Path is not stored correctly')

        # Check report name
        self.assertEqual(report.report_name,
                         'Almapiwrapper_test')

        # Check content
        self.assertGreater(len(report.data), 6, 'Analytics report contains not enough data')
        self.assertEqual(report.data.iloc[0, 0],
                         '41SLSP_NETWORK',
                         'Analytics report contains not the expected data (41SLSP_NETWORK')

    def test_get_analytics_report_with_filter(self):
        f = '''<sawx:expr xsi:type="sawx:comparison" op="equal"
                xmlns:saw="com.siebel.analytics.web/report/v1.1" 
                xmlns:sawx="com.siebel.analytics.web/expression/v1.1" 
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
                xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                  <sawx:expr xsi:type="sawx:sqlExpression">"Title Creation Date"."Title Creation Year"</sawx:expr>
                  <sawx:expr xsi:type="xsd:string">2023</sawx:expr>
               </sawx:expr>'''
        report = AnalyticsReport('/shared/SLSP Network Zone 41SLSP_NETWORK/Reports/RRE/Almapiwrapper_filter_test', 'NZ', 'P', filter_to_apply=f)

        # Check path
        self.assertEqual(report.path,
                         '/shared/SLSP Network Zone 41SLSP_NETWORK/Reports/RRE/Almapiwrapper_filter_test',
                         'Path is not stored correctly')

        # Check report name
        self.assertEqual(report.report_name,
                         'Almapiwrapper_filter_test')

        # Check content
        self.assertEqual(len(report.data), 1, 'Analytics report should contain 1 row')
        self.assertEqual(report.data.iloc[0, 0],
                         '41SLSP_NETWORK',
                         'Analytics report contains not the expected data (41SLSP_NETWORK')


if __name__ == '__main__':
    unittest.main()



from ..record import Record, check_error, JsonData
from typing import Optional, Literal,ClassVar, List, Union
import almapiwrapper.acquisitions as acquisitionslib
from lxml import etree
import requests
import logging
from json import JSONDecodeError


def _handle_error(q: str, r: requests.models.Response, msg: str, zone: str, env: str):
    """Set the handle errors of the fetch_invoices function

    :param q: str : query to fetch the invoices
    :param r: request response of the api
    :param msg: context message of the error
    :return: None
    """
    try:
        json_data = r.json()
        error_message = json_data['errorList']['error'][0]['errorMessage']
    except JSONDecodeError:
        xml = etree.fromstring(r.content)
        error_message = xml.find('.//{http://com/exlibris/urm/general/xmlbeans}errorMessage').text

    logging.error(f'fetch_invoices("{q}", "{zone}", "{env}") - {r.status_code}: '
                  f'{msg} / {error_message}')

class Invoice(Record):
    """Class representing an Invoice

    :ivar invoice_id: initial value: code of the vendor
    :param invoice_number: initial value: invoice_number of the invoice
    :ivar zone: initial value: zone of the vendor
    :ivar env: initial value: environment of the entity: 'P' for production and 'S' for sandbox
    :ivar data: :class:`almapiwrapper.record.JsonData` with raw vendor data
    """

    api_base_url_invoices: ClassVar[str] = f'{Record.api_base_url}/acq/invoices'

    def __init__(self,
                 invoice_id: Optional[str] = None,
                 zone: Optional[str] = None,
                 env: Literal['P', 'S'] = 'P',
                 invoice_number: Optional[str] = None,
                 data: Optional[Union[dict, JsonData]] = None) -> None:
        """Constructor of Invoice Object
        """

        super().__init__(zone, env)
        self.area = 'Acquisitions'
        self.format = 'json'
        self.invoice_id = invoice_id
        if data is not None:
            self._data = data if type(data) is JsonData else JsonData(data)
        elif invoice_number is not None:
            invoices = fetch_invoices(f'invoice_number~{invoice_number}', zone, env)
            if len(invoices) > 0:
                self._data = JsonData(invoices[0].data)
                self.invoice_id = invoices[0].invoice_id
            else:
                self.error = True
                logging.error(f'No invoice found for invoice_number: {invoice_number}')

        elif invoice_id is not None:
            pass
        else:
            self.error = True
            logging.error('Missing information to construct an Invoice')

    def __repr__(self) -> str:
        """Get a string representation of the object. Useful for logs.

        :return: str representing the object
        """
        return f"{self.__class__.__name__}('{self.invoice_id}', '{self.zone}', '{self.env}')"

    @property
    def invoice_number(self) -> str:
        """invoice_number(self) -> str
        Get the invoice invoice_number

        :return: str
        """
        return self.data['number']

    @invoice_number.setter
    @check_error
    def invoice_number(self, invoice_number: str) -> None:
        """invoice_number(self, barcode: str) -> None
        This setter is able to update the invoice number of the item. But the field should already exist.

        :param invoice_number: barcode of the item

        :return: None
        """
        if 'number' not in self.data:
            logging.error(f'{repr(self)}: no number field in the invoice -> not possible to update it')
            self.error = True
        else:
            logging.info(f'{repr(self)}: invoice number changed from "{self.data["number"]}" to "{invoice_number}"')
            self.data['number'] = invoice_number

    def _fetch_data(self) -> Optional[JsonData]:
        """Fetch the json data of the invoice

        :return: :class:`almapiwrapper.record.JsonData` if no error else None
        """

        r = self._api_call('get',
                     f'{self.api_base_url_invoices}/{self.invoice_id}',
                           headers=self._get_headers())
        if r.ok is True:
            # Parse data
            json_data = JsonData(r.json())
            logging.debug(f"{self.__class__.__name__} data fetched")

            return json_data
        else:
            self._handle_error(r, 'unable to fetch invoice data')

    @check_error
    def update(self) -> 'acquisitionslib.Invoice':
        """update(self) -> 'acquisitionslib.Invoice'
        Update the Invoice

        .. note::
            If the record encountered an error, this
            method will be skipped.

        :return: Vendor object
        """

        r = self._api_call('put',
                           f'{self.api_base_url_invoices}/{self.invoice_id}',
                           headers=self._get_headers(),
                           data=bytes(self))

        if r.ok is True:
            self.data = JsonData(r.json())
            logging.info(f'{repr(self)}: Invoice data updated')
        else:
            self._handle_error(r, 'unable to update invoice data')

        return self

    @check_error
    def save(self) -> 'acquisitionslib.Invoice':
        """save(self) -> acquisitionslib.Invoice
        Save a PO Line record in the 'records' folder

        When saved, a suffix is added to the file path with the version.
        Example: records/vendors/vendor_<IZ>_<set_id>_<version>.xml

        :return: object :class:`almapiwrapper.acquisitions.Invoice`

        .. note::
            If the record encountered an error, this
            method will be skipped.
        """
        filepath = f'records/invoices/invoice_{self.zone}_{self.invoice_id}.json'
        self._save_from_path(filepath)
        return self

    @check_error
    def create(self) -> 'acquisitionslib.Invoice':
        """create(self) -> acquisitionslib.Invoice
        Create a Invoice

        :return: Invoice object"""
        r = self._api_call('post',
                           f'{self.api_base_url_invoices}',
                           headers=self._get_headers(),
                           data=bytes(self))

        if r.ok is True:
            self._data = JsonData(r.json())
            self.invoice_id = self.data['id']
            logging.info(f'{repr(self)}: Invoice created: {self.invoice_id}')
        else:
            self._handle_error(r, 'unable to create Invoice')

        return self

    @check_error
    def process_invoice(self, op='process_invoice') -> 'acquisitionslib.Invoice':
        """process_invoice(self) -> acquisitionslib.Invoice
        Process the invoice

        This operation is useful after adding invoice lines to the invoice

        :return: object :class:`almapiwrapper.acquisitions.Invoice`
        """
        r = self._api_call('post',
                           f'{self.api_base_url_invoices}/{self.invoice_id}',
                           headers=self._get_headers(),
                           data=bytes(self),
                           params={'op': op})

        if r.ok is True:
            self.data = JsonData(r.json())
            logging.info(f'{repr(self)}: Invoice processed')
        else:
            self._handle_error(r, 'unable to process invoice')

        return self

    def get_invoice_lines(self, limit: Optional[int] = None) -> List['InvoiceLine']:
        """get_invoice_lines(self, limit: Optional[int] = None) -> List['InvoiceLine']
        Get the invoice lines of the invoice

        :param limit: int : limit of invoice lines to fetch

        :return: list of invoice lines
        """
        if 'invoice_lines' not in self.data:
            logging.warning(f'{repr(self)}: No invoice lines found')
            return []
        invoice_lines = [InvoiceLine(data=invoice_line_data, invoice_id=self.invoice_id, zone=self.zone, env=self.env)
                         for invoice_line_data in self.data['invoice_lines']['invoice_line']]
        return invoice_lines


def fetch_invoices(q: str,
                   zone: str,
                   env: Literal['P', 'S'] = 'P',
                   limit: Optional[int] = None) -> List['Invoice']:
    """fetch_invoices(q: str, zone: str, env: Literal['P', 'S'] = 'P', limit: Optional[int] = None) -> List['Invoices']
    Fetch invoices from a query

    :param q: str : query to fetch the invoices
    :param zone: str : zone of the invoices
    :param env: Literal['P', 'S'] : environment of the entity: 'P' for production and 'S' for sandbox
    :param limit: int : limit of invoices to fetch

    :return: list of invoices
    """
    params = {'q': q}

    if limit is not None:
        params['limit'] = limit

    r = Record._api_call('get',
                         f'{Record.api_base_url}/acq/invoices',
                         headers=Record.build_headers(data_format='json', env=env,
                                                      zone=zone, rights='RW', area='Acquisitions'),
                         params=params)
    if r.ok is True:
        json_data = JsonData(r.json())
        if 'invoice' in json_data.content and json_data.content['invoice'] is not None:
            invoices = [Invoice(invoice['id'], zone=zone, env=env, data=invoice)
                        for invoice in json_data.content['invoice']]
            logging.info(f'fetch_invoices("{q}", "{zone}", "{env}"): '
                         f'{len(invoices)} invoices data available')

            return invoices
        else:
            logging.warning(f'fetch_invoices("{q}", "{zone}", "{env}"): '
                            f'0 invoices data available')
            return []
    else:
        _handle_error(q, r, 'unable to fetch invoices', zone, env)


class InvoiceLine(Record):
    """Class representing an InvoiceLine"""

    api_base_url_invoices: ClassVar[str] = f'{Record.api_base_url}/acq/invoices'

    def __init__(self,
                 invoice_id: Optional[str] = None,
                 invoice_line_id: Optional[str] = None,
                 zone: Optional[str] = None,
                 env: Literal['P', 'S'] = 'P',
                 invoice_number: Optional[str] = None,
                 data: Optional[Union[dict, JsonData]] = None) -> None:
        """Constructor of InvoiceLine Object
        """

        super().__init__(zone, env)
        self.area = 'Acquisitions'
        self.format = 'json'
        self.invoice_line_id = invoice_line_id
        self.invoice_id = invoice_id
        self.invoice_number = invoice_number

        # Invoice id and invoice line id are provided we can directly fetch the data
        if invoice_line_id is not None and invoice_id is not None:
            self._data = self._fetch_data()

        if data is not None:
            self._data = data if type(data) is JsonData else JsonData(data)
            if 'id' in self.data:
                self.invoice_line_id = self.data['id']

        if self.invoice_number is not None:
            invoice = Invoice(invoice_number=self.invoice_number, zone=zone, env=env)
            _ = invoice.data
            if invoice.error is False:
                self.invoice_id = invoice.invoice_id
            else:
                self.error = True
                logging.error(f'No invoice found for invoice_number: {invoice_number}')


        if data is None and (self.invoice_line_id is None or self.invoice_id is None):
            self.error = True
            logging.error('Missing information to construct an InvoiceLine')

    def __repr__(self) -> str:
        """Get a string representation of the object. Useful for logs.

        :return: str representing the object
        """
        return f"{self.__class__.__name__}('{self.invoice_id}', '{self.invoice_line_id}', '{self.zone}', '{self.env}')"


    def _fetch_data(self) -> Optional[JsonData]:
        """Fetch the json data of the invoice

        :return: :class:`almapiwrapper.record.JsonData` if no error else None
        """

        r = self._api_call('get',
                     f'{self.api_base_url_invoices}/{self.invoice_id}/lines/{self.invoice_line_id}',
                           headers=self._get_headers())
        if r.ok is True:
            # Parse data
            json_data = JsonData(r.json())
            logging.debug(f"{self.__class__.__name__} data fetched")

            return json_data
        else:
            self._handle_error(r, 'unable to fetch vendor data')

    @check_error
    def update(self) -> 'acquisitionslib.InvoiceLine':
        """update(self) -> 'acquisitionslib.InvoiceLine'
        Update the InvoiceLine

        .. note::
            If the record encountered an error, this
            method will be skipped.

        :return: Vendor object
        """

        r = self._api_call('put',
                           f'{self.api_base_url_invoices}/{self.invoice_id}/lines/{self.invoice_line_id}',
                           headers=self._get_headers(),
                           data=bytes(self))

        if r.ok is True:
            self.data = JsonData(r.json())
            logging.info(f'{repr(self)}: InvoiceLine data updated')
        else:
            self._handle_error(r, 'unable to update invoice line data')

        return self

    @check_error
    def create(self) -> 'acquisitionslib.InvoiceLine':
        """add_to_invoice(self) -> acquisitionslib.InvoiceLine
        Add the invoice line to the invoice

        :return: object :class:`almapiwrapper.acquisitions.InvoiceLine`
        """
        data = self.data
        for fund in data['fund_distribution']:
            if 'amount' in fund and 'percent' in fund:
                del fund['percent']

        r = self._api_call('post',
                           f'{self.api_base_url_invoices}/{self.invoice_id}/lines',
                           headers=self._get_headers(),
                           data=bytes(self))

        if r.ok is True:
            self.data = JsonData(r.json())
            self.invoice_line_id = self.data['id']
            logging.info(f'{repr(self)}: InvoiceLine added to invoice')
        else:
            self._handle_error(r, 'unable to add invoice line to invoice')

        return self